﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PTMedicalInsurance.Entity
{
    /// <summary>
    /// 【2301】住院费用明细上传
    /// 交易说明：通过此交易进行住院费用明细上传、明细退单。
    /// </summary>
    public class InpatientExpenseDetailUploadInput
    {
        /// <summary>
        /// 费用明细流水号
        /// 单次就诊内唯一
        /// </summary>
        public string feedetl_sn { get; set; }

        /// <summary>
        /// 原费用流水号
        /// 退单时传入被退单的费用明细流水号
        /// </summary>
        public string init_feedetl_sn { get; set; }

        /// <summary>
        /// 就诊 ID
        /// </summary>
        public string mdtrt_id { get; set; }

        /// <summary>
        /// 医嘱号
        /// </summary>
        public string drord_no { get; set; }

        /// <summary>
        /// 人员编号
        /// </summary>
        public string psn_no { get; set; }

        /// <summary>
        /// 医疗类别
        /// </summary>
        public string med_type { get; set; }

        /// <summary>
        /// 费用发生时间
        /// </summary>
        public DateTime fee_ocur_time { get; set; }

        /// <summary>
        /// 医疗目录编码
        /// </summary>
        public string med_list_codg { get; set; }

        /// <summary>
        /// 医药机构目录编码
        /// </summary>
        public string medins_list_codg { get; set; }

        /// <summary>
        /// 明细项目费用总额
        /// </summary>
        public decimal det_item_fee_sumamt { get; set; }

        /// <summary>
        /// 数量
        /// 退单时数量填写负数
        /// </summary>
        public decimal cnt { get; set; }

        /// <summary>
        /// 单价
        /// </summary>
        public decimal pric { get; set; }

        /// <summary>
        /// 开单科室编码
        /// </summary>
        public string bilg_dept_codg { get; set; }

        /// <summary>
        /// 开单科室名称
        /// </summary>
        public string bilg_dept_name { get; set; }

        /// <summary>
        /// 开单医生编码
        /// </summary>
        public string bilg_dr_codg { get; set; }

        /// <summary>
        /// 开单医师姓名
        /// </summary>
        public string bilg_dr_name { get; set; }

        /// <summary>
        /// 受单科室编码
        /// </summary>
        public string acord_dept_codg { get; set; }

        /// <summary>
        /// 受单科室名称
        /// </summary>
        public string acord_dept_name { get; set; }

        /// <summary>
        /// 受单医生编码
        /// </summary>
        public string orders_dr_codg { get; set; }

        /// <summary>
        /// 受单医生姓名
        /// </summary>
        public string orders_dr_name { get; set; }

        /// <summary>
        /// 医院审批标志
        /// </summary>
        public string hosp_appr_flag { get; set; }

        /// <summary>
        /// 中药使用方式
        /// </summary>
        public string tcmdrug_used_way { get; set; }

        /// <summary>
        /// 外检标志
        /// </summary>
        public string etip_flag { get; set; }

        /// <summary>
        /// 外检医院编码
        /// </summary>
        public string etip_hosp_code { get; set; }

        /// <summary>
        /// 出院带药标志
        /// </summary>
        public string dscg_tkdrug_flag { get; set; }

        /// <summary>
        /// 生育费用标志
        /// </summary>
        public string matn_fee_flag { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        public string memo { get; set; }

        /// <summary>
        /// 扩展参数
        /// 用于扩展出更多的参数，格式为 JSON 格式的字符串
        /// </summary>
        public string exp_content { get; set; }
    }

    /// <summary>
    /// 【2301】住院费用明细上传
    /// 交易说明：通过此交易进行住院费用明细上传、明细退单。
    /// </summary>
    public class InpatientExpenseDetailUploadOutput
    {
        /// <summary>
        /// 费用明细流水号
        /// </summary>
        public string feedetl_sn { get; set; }

        /// <summary>
        /// 明细项目费用总额
        /// </summary>
        public decimal det_item_fee_sumamt { get; set; }

        /// <summary>
        /// 数量
        /// </summary>
        public decimal cnt { get; set; }

        /// <summary>
        /// 单价
        /// </summary>
        public decimal pric { get; set; }

        /// <summary>
        /// 定价上限金额
        /// </summary>
        public decimal pric_uplmt_amt { get; set; }

        /// <summary>
        /// 自付比例
        /// </summary>
        public decimal selfpay_prop { get; set; }

        /// <summary>
        /// 全自费金额
        /// </summary>
        public decimal fulamt_ownpay_amt { get; set; }

        /// <summary>
        /// 超限价金额
        /// </summary>
        public decimal overlmt_amt { get; set; }

        /// <summary>
        /// 先行自付金额
        /// </summary>
        public decimal preselfpay_amt { get; set; }

        /// <summary>
        /// 符合政策范围金额
        /// </summary>
        public decimal inscp_scp_amt { get; set; }

        /// <summary>
        /// 收费项目等级
        /// </summary>
        public string chrgitm_lv { get; set; }

        /// <summary>
        /// 医疗收费项目类别
        /// </summary>
        public string med_chrgitm_type { get; set; }

        /// <summary>
        /// 基本药物标志
        /// </summary>
        public string bas_medn_flag { get; set; }

        /// <summary>
        /// 医保谈判药品标志
        /// </summary>
        public string hi_nego_drug_flag { get; set; }

        /// <summary>
        /// 儿童用药标志
        /// </summary>
        public string chld_medc_flag { get; set; }

        /// <summary>
        /// 目录特项标志
        /// </summary>
        public string list_sp_item_flag { get; set; }

        /// <summary>
        /// 限制使用标志
        /// </summary>
        public string lmt_used_flag { get; set; }

        /// <summary>
        /// 直报标志
        /// </summary>
        public string drt_reim_flag { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        public string memo { get; set; }
    }

}
